﻿using Alarm;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks; // .NET Franework 3.5 では使えないのでコメントアウト
using System.Windows.Forms;
using System.Runtime.InteropServices;   // DllImportのために追加
using System.Diagnostics; // ウィンドウプロセス取得のために追加

namespace MyClock
{
    public partial class Display : Form
    {
        public Display()
        {
            InitializeComponent();
        }

        // Form生成のメッセージハンドラ
        private void Display_Load(object sender, EventArgs e)
        {
            DisplayTime();

            // マウスのメッセーハンドラ作成
            currentTime.MouseDown += new MouseEventHandler(Display_MouseDown);
            currentTime.MouseMove += new MouseEventHandler(Display_MouseMove);
        }

        // Timerのメッセージハンドラ
        private void secTimer_Tick(object sender, EventArgs e)
        {
            DisplayTime();
        }

        // 現在時刻の表示
        private void DisplayTime()
        {
            string time = DateTime.Now.ToString("HH:mm:ss");
            currentTime.Text = time;

            // アラーム設定時間になったら時計の外観を変えて点滅させる
            if (time == m_Alarm)
            {
                if (!m_cancelAlarm)
                {
                    ChangeDisplay();
                    FlashWnd();
                }
            }
        }

        // ラベルをシングルクリックした時のイベントハンドラ
        // アラーム状態のウィンドウを元に戻す
        private void currentTime_Click(object sender, EventArgs e)
        {
            ResumeDisplay();
            StopFlashWnd();
        }

        // Labelのマウス左ボタンダブルクリックのメッセーハンドラ
        private void currentTime_DoubleClick(object sender, EventArgs e)
        {
            //プログラムを終了する
            Dispose();
        }

        //マウスのクリック位置を記憶する変数
        private Point mousePoint;

        //DisplayのMouseDownイベントハンドラ
        //マウスのボタンが押されたとき
        private void Display_MouseDown(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                //カーソルをHandに変更
                this.Cursor = Cursors.Hand;
                //位置を記憶する
                mousePoint = new Point(e.X, e.Y);
            }
            else
            {
                //カーソルを元に戻す
                this.Cursor = Cursors.Default;
            }

            if ((e.Button & MouseButtons.Right) == MouseButtons.Right)
            {
                //最小化する
                this.WindowState = FormWindowState.Minimized;
            }
        }

        //DisplayのMouseMoveイベントハンドラ
        //マウスが動いたとき
        private void Display_MouseMove(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                //カーソルをHandに変更
                this.Cursor = Cursors.Hand;
                //フォームの位置変更
                this.Left += e.X - mousePoint.X;
                this.Top += e.Y - mousePoint.Y;
            }
            else
            {
                //カーソルを元に戻す
                this.Cursor = Cursors.Default;
            }
        }

        // アラーム時間の格納変数
        public static string m_Alarm;

        // アラーム解除フラグ
        public static bool m_cancelAlarm = false;

        // [Esc]キーを押した時のイベントハンドラ
        // アラーム設定
        private void Display_KeyDown(object sender, KeyEventArgs e)
        {
            // アラーム動作確認用
            /*if (e.KeyCode == Keys.Tab)
            {
                ChangeDisplay();
                FlashWnd();
            }*/
            //動作確認用ここまで

            if (e.KeyCode == Keys.Escape)
            {
                Alarm.Alarm.m_alarmCanceled = m_cancelAlarm;
                if (m_Alarm != null)
                    Alarm.Alarm.m_alarmTime = m_Alarm;

                // アラーム設定フォームのインスタンス作成
                Alarm.Alarm dlg = new Alarm.Alarm();
                dlg.StartPosition = FormStartPosition.CenterParent;
                // アラーム設定フォームをモーダルダイアログで表示
                if (dlg.ShowDialog(this) == DialogResult.OK)
                {
                    // OK が返ってきたときの処理
                    m_cancelAlarm = Alarm.Alarm.m_alarmCanceled;
                    string m_time = Alarm.Alarm.m_alarmTime;
                    if (m_time != null || m_time != m_Alarm)
                    {
                            // アラーム設定時刻が変更された場合、static m_Alarm 変数を更新
                            m_Alarm = Alarm.Alarm.m_alarmTime;
                            //MessageBox.Show(" " + m_Alarm + " でいいですか？", "アラーム設定",
                            //               MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    }
                }
                // MSDNでダイアログの破棄をすべきと推奨している
                dlg.Dispose();
            }
        }

        // Win32 APIを使うための準備
        [DllImport("user32.dll")]
        static extern Int32 FlashWindowEx(ref FLASHWINFO pwfi);

        // FLASHWININFO 構造体のオーバーライド
        [StructLayout(LayoutKind.Sequential)]
        public struct FLASHWINFO
        {
            public UInt32 cbSize;           // FLASHWINFO構造体のサイズ
            public IntPtr hwnd;              // 点滅対象のウィンドウ・ハンドル
            public UInt32 dwFlags;        // 以下の「FLASHW_XXX」のいずれか
            public UInt32 uCount;         // 点滅する回数
            public UInt32 dwTimeout;   // 点滅する間隔（ミリ秒単位）
        }

        // FLASHWININFO メンバの実装
        // 点滅を止める
            public const UInt32 FLASHW_STOP = 0;
        // タイトルバーを点滅させる
            public const UInt32 FLASHW_CAPTION = 1;
        // タスクバー・ボタンを点滅させる
            public const UInt32 FLASHW_TRAY = 2;
        // タスクバー・ボタンとタイトルバーを点滅させる
            public const UInt32 FLASHW_ALL = 3;
        // FLASHW_STOPが指定されるまでずっと点滅させる
            public const UInt32 FLASHW_TIMER = 4;
        // ウィンドウが最前面に来るまでずっと点滅させる
            public const UInt32 FLASHW_TIMERNOFG = 12;

        // 時計の見え方を変える
        private void ChangeDisplay()
        {
            // ウィンドウスタイルを変更する
            this.FormBorderStyle = FormBorderStyle.FixedToolWindow;
            this.Opacity = 1D;
            this.Text = "It's Time! - Click Here";
        }

        // 時計の見え方を元に戻す
        private void ResumeDisplay()
        {
            // ウィンドウスタイルを変更する
            this.FormBorderStyle = FormBorderStyle.None;
            this.Opacity = 0.5D;
            this.Text = "MyClock";
        }

        // ウィンドウを点滅させる
        private void FlashWnd()
        {
            FLASHWINFO flsh = new FLASHWINFO();
            flsh.cbSize = Convert.ToUInt32(Marshal.SizeOf(flsh));
            flsh.hwnd = this.Handle;
            flsh.dwFlags = FLASHW_ALL | FLASHW_TIMER;
            //flsh.uCount = 5;
            flsh.dwTimeout = 100;

            FlashWindowEx(ref flsh);
        }

        // ウィンドウの点滅を止める
        private void StopFlashWnd()
        {
            FLASHWINFO flsh = new FLASHWINFO();
            flsh.cbSize = Convert.ToUInt32(Marshal.SizeOf(flsh));
            flsh.hwnd = this.Handle;
            flsh.dwFlags = FLASHW_STOP;

            FlashWindowEx(ref flsh);
        }
    }
}
